@@ -1,4 +1,4 @@
-trimkmeans <- function(data,k,trim=0.1, scaling=FALSE, runs=100, points=NULL,
+trimkmeans <- function(data, k, trim=0.1, scaling=FALSE, runs=100, points=NULL,
                        countmode=runs+1, printcrit=FALSE,
                        maxit=2*nrow(as.matrix(data))){
   data <- as.matrix(data)
@@ -8,13 +8,17 @@ trimkmeans <- function(data,k,trim=0.1, scaling=FALSE, runs=100, points=NULL,
   crit <- Inf
   oldclass <- iclass <- optclass <- rep(0,n)
   disttom <- rep(0,n)
+
 #  optmeans <- data[sample(n,k),,drop=FALSE]
-  for (i in 1:runs){
-    if ((i/countmode)==round(i/countmode)) cat("Iteration ",i,"\n")
-    if (is.null(points))
+  for(i in 1:runs){
+    if((i/countmode) == round(i/countmode)) 
+        cat("Iteration ",i,"\n")
+    
+    if(is.null(points))
       means <- data[sample(n,k),,drop=FALSE]
     else
       means <- points
+      
     wend <- FALSE
     itcounter <- 0
     while(!wend){
@@ -26,11 +30,16 @@ trimkmeans <- function(data,k,trim=0.1, scaling=FALSE, runs=100, points=NULL,
         iclass[j] <- which.min(dj)
         disttom[j] <- min(dj)
       }      
-      iclass[order(disttom)[(nin+1):n]] <- 0
+      
+      ## VT::27.01.2024
+      if(nin < n)
+        iclass[order(disttom)[(nin+1):n]] <- 0
+
 #    newcrit <- sum(disttom[iclass>0])
 #    cat("Iteration ",i," criterion value ",newcrit,"\n")
+
       if (itcounter>=maxit | identical(oldclass,iclass)) wend <- TRUE
-      else{
+      else {
         for (l in 1:k){
           if (sum(iclass==l)==0) means[l,] <- data[iclass==0,,drop=FALSE][1,]
           else{
@@ -46,19 +55,22 @@ trimkmeans <- function(data,k,trim=0.1, scaling=FALSE, runs=100, points=NULL,
         oldclass <- iclass
       }
     }
+    
     newcrit <- sum(disttom[iclass>0])
-    if (printcrit) cat("Iteration ",i," criterion value ",
-                       newcrit/nin,"\n")
-    if (newcrit<=crit){
+    if(printcrit) cat("Iteration ",i," criterion value ", newcrit/nin,"\n")
+    if(newcrit <= crit){
       optclass <- iclass
       crit <- newcrit
       optmeans <- means
     }
   }
+
   optclass[optclass==0] <- k+1
-  out <- list(classification=optclass,means=optmeans,
-              criterion=crit/nin,disttom=disttom,ropt=sort(disttom)[nin],
-              k=k,trim=trim,runs=runs,scaling=scaling)
+
+  out <- list(classification=optclass, means=optmeans,
+              criterion=crit/nin, disttom=disttom, ropt=sort(disttom)[nin],
+              k=k, trim=trim, runs=runs, scaling=scaling)
+  
   class(out) <- "tkm"
   out
 }
